library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

--produce an N bit pulse train @ 40 kHz to the transmitting transducer
--for our application we have two output pulse signals, and two enable signals
--which are based on the data written, but this can be modified so that there
--is only one transmit pulse signal, and zero enable signals.
--Also, for our application, all outputs are active low; the output pulses will be asserted
--low.

entity tx_enable is
	Port(
		avs_data_writedata: in std_logic_vector(31 downto 0) := (others => '0');     --input data for pulse info
		avs_data_write_n: in std_logic; 											 --write_n signal for data writes
		coe_TxA_export: out std_logic;												 --transmit A pulses
		coe_TxB_export: out std_logic;												 --transmit B pulses
		coe_AN_export: out std_logic;											     --A_N enable signal for transducer A
		coe_BN_export: out std_logic;												 --B_N enable signal for transducer B
		csi_clockreset_clk: in std_logic;											 --input clock (50 MHz)
		csi_clockreset_reset_n: in std_logic										 --input clock reset
	);
end entity tx_enable;

architecture states of tx_enable is

	type state_type is (wait_for_write, clock_pulse_out);
	signal state: state_type;
	signal dataHold: std_logic_vector(31 downto 0);
	signal AN: std_logic:= '1';													--active low(for active high = '0')
	signal BN: std_logic:= '1';													--active low(for active high = '0')
	signal pulse_length: std_logic_vector(3 downto 0);
	signal hilo: std_logic := '1';											    --active low(for active high = '0'), this is the signal that will be assigned to our pulse output
	signal ready: std_logic := '1';
	
begin
	enable: process(csi_clockreset_clk)
		variable pulseLength: integer;
		variable freq_count: integer:= 0;										--counter to regulate pulse train length
		variable pulse_count: integer := 0;										--counter to keep track of pulses transmitted
		
		variable factor_low: integer := 625;									--factor_low and factor_high are used to control
																				--the frequency of the pulses we transmit, since our
		variable factor_high: integer := 1250;									--the state machine is dependant on each rising edge
																				--of the 50 MHz clock, these factors are determined to ensure
																				--that the frequency of the transmitted pulses are satisfactory
																				--(for us we want 40 KHz) 50 MHz/40 kHz = 1250. So each output
																				--pulse will be low for 625 system clock cycles, and then high for
																				--the next 625.
	begin
		if (avs_data_write_n = '1' and ready = '1') then						--wait for write_n to assert to signal that a write occured
			state <= wait_for_write;
		elsif rising_edge(csi_clockreset_clk) then
			case state is
				when wait_for_write=>
					ready <= '0';												--ensure that we do not enter the first IF statement
					dataHold<= (others => '0');									--reset the dataHold signal to 0 on every write
					if (avs_data_write_n = '0') then
						dataHold <= avs_data_writedata;							--fill dataHold signal with the data that was written
						state <= clock_pulse_out;								
					end if;
				when clock_pulse_out=>
						AN <= not dataHold(0);									--based on the input data we assign bits 0 and 1 to our enable signals
						BN <= not dataHold(1);									--***this can be ommitted if enable signals are not required***
						
						if (AN = BN) then 										--ensure that the data written is not the same(for our application
							hilo <= '1';										--we only want one enable asserted at once)
						else
							pulse_length <= dataHold(5 downto 2);				--based on the written data, a 4 bit value determines the length of the pulse train
							pulseLength := to_integer(unsigned(pulse_length));
			
							if (pulse_count < pulseLength + 1) then				--check that we haven't gone over our desired pulse train length

								freq_count := freq_count+1;
				
								if(freq_count = factor_low or freq_count < factor_low) then	--keep our output signal low until we have reached factor low
									hilo <= '1';
								end if;
				
								if(freq_count = factor_high) then					--at this point one period will have been outputted
									hilo <= '1';									--hilo returns to '1'(or '0' for active high signals)
									freq_count := 0;								--reset the frequency counter for the next pulse
									pulse_count := pulse_count+1;					--add one to our pulse train counter
								end if;
				
								if(freq_count > factor_low) then					--once we have reached factor low, we want to set hilo to '0'
									hilo <= '0';									--this is the beginning of our low output pulse(for active high
								end if;												--pulses this would be set to '1' at this point.
							else
								pulse_count := 0;									--here we have reached our pulse train limit and want to reset everything
								dataHold <= (others => '0');
								AN <= '1';
								BN <= '1';
								ready <= '1';
							end if;
						end if;
			end case;
		end if;
	end process enable;
   coe_AN_export<=AN; 						
	coe_BN_export<=BN;							
	coe_TxA_export<=hilo or AN;					--this OR is done to ensure that our un-used transmit signals stay high
	coe_TxB_export<=hilo or BN;					--it will not be necessary for all applications.
end architecture states;